@@ -12,9 +12,11 @@ from charmhelpers.core import host
 class NRPEBaseTestCase(TestCase):
     patches = {
         'config': {'object': nrpe},
+        'copy2': {'object': nrpe.shutil},
         'log': {'object': nrpe},
         'getpwnam': {'object': nrpe.pwd},
         'getgrnam': {'object': nrpe.grp},
+        'glob': {'object': nrpe.glob},
         'mkdir': {'object': os},
         'chown': {'object': os},
         'chmod': {'object': os},
@@ -23,7 +25,9 @@ class NRPEBaseTestCase(TestCase):
         'remove': {'object': os},
         'open': {'object': nrpe, 'create': True},
         'isfile': {'object': os.path},
+        'isdir': {'object': os.path},
         'call': {'object': subprocess},
+        'relation_get': {'object': nrpe},
         'relation_ids': {'object': nrpe},
         'relation_set': {'object': nrpe},
         'relations_of_type': {'object': nrpe},
@@ -120,6 +124,11 @@ class NRPETestCase(NRPEBaseTestCase):
         self.patched['config'].return_value = {'nagios_context': 'a',
                                                'nagios_servicegroups': ''}
         self.patched['exists'].return_value = True
+        self.patched['relation_get'].return_value = {
+            'egress-subnets': '10.66.111.24/32',
+            'ingress-address': '10.66.111.24',
+            'private-address': '10.66.111.24'
+        }
 
         def _rels(rname):
             relations = {
@@ -178,8 +187,8 @@ define service {
         ]
         self.patched['relation_set'].assert_has_calls(relation_set_calls, any_order=True)
         self.check_call_counts(config=1, getpwnam=1, getgrnam=1,
-                               exists=4, open=2, listdir=1,
-                               relation_ids=4, relation_set=3)
+                               exists=4, open=2, listdir=1, relation_get=2,
+                               relation_ids=3, relation_set=3)
 
 
 class NRPECheckTestCase(NRPEBaseTestCase):
@@ -187,7 +196,6 @@ class NRPECheckTestCase(NRPEBaseTestCase):
     def test_invalid_shortname(self):
         cases = [
             'invalid:name',
-            '@invalid',
             '',
         ]
         for shortname in cases:
@@ -201,6 +209,7 @@ class NRPECheckTestCase(NRPEBaseTestCase):
             'dashes-ok',
             'UPPER_case_allowed',
             '5',
+            '@valid',
         ]
         for shortname in cases:
             check = nrpe.Check(shortname, 'description', '/some/command')
@@ -347,7 +356,8 @@ class NRPEMiscTestCase(NRPEBaseTestCase):
             stderr=subprocess.STDOUT)
 
         # Test regular services and snap services with systemd
-        services = ['apache2', 'haproxy', 'snap.test.test']
+        services = ['apache2', 'haproxy', 'snap.test.test',
+                    'ceph-radosgw@hostname']
         self.patched['init_is_systemd'].return_value = True
         nrpe.add_init_service_checks(bill, services, 'testunit')
         expect_cmds = {
@@ -361,3 +371,49 @@ class NRPEMiscTestCase(NRPEBaseTestCase):
         self.assertEqual(bill.checks[3].check_cmd, expect_cmds['haproxy'])
         self.assertEqual(bill.checks[4].shortname, 'snap.test.test')
         self.assertEqual(bill.checks[4].check_cmd, expect_cmds['snap.test.test'])
+
+    def test_copy_nrpe_checks(self):
+        file_presence = {
+            'filea': True,
+            'fileb': False}
+        self.patched['exists'].return_value = True
+        self.patched['glob'].return_value = ['filea', 'fileb']
+        self.patched['isdir'].side_effect = [False, True]
+        self.patched['isfile'].side_effect = lambda x: file_presence[x]
+        nrpe.copy_nrpe_checks()
+        self.patched['glob'].assert_called_once_with(
+            ('/usr/lib/test_charm_dir/hooks/charmhelpers/contrib/openstack/'
+             'files/check_*'))
+        self.patched['copy2'].assert_called_once_with(
+            'filea',
+            '/usr/local/lib/nagios/plugins/filea')
+
+    def test_copy_nrpe_checks_other_root(self):
+        file_presence = {
+            'filea': True,
+            'fileb': False}
+        self.patched['exists'].return_value = True
+        self.patched['glob'].return_value = ['filea', 'fileb']
+        self.patched['isdir'].side_effect = [True, False]
+        self.patched['isfile'].side_effect = lambda x: file_presence[x]
+        nrpe.copy_nrpe_checks()
+        self.patched['glob'].assert_called_once_with(
+            ('/usr/lib/test_charm_dir/charmhelpers/contrib/openstack/'
+             'files/check_*'))
+        self.patched['copy2'].assert_called_once_with(
+            'filea',
+            '/usr/local/lib/nagios/plugins/filea')
+
+    def test_copy_nrpe_checks_nrpe_files_dir(self):
+        file_presence = {
+            'filea': True,
+            'fileb': False}
+        self.patched['exists'].return_value = True
+        self.patched['glob'].return_value = ['filea', 'fileb']
+        self.patched['isfile'].side_effect = lambda x: file_presence[x]
+        nrpe.copy_nrpe_checks(nrpe_files_dir='/other/dir')
+        self.patched['glob'].assert_called_once_with(
+            '/other/dir/check_*')
+        self.patched['copy2'].assert_called_once_with(
+            'filea',
+            '/usr/local/lib/nagios/plugins/filea')
