@@ -328,6 +328,81 @@ class CephUtilsTests(TestCase):
                              valid_type=str,
                              valid_range=["foo"])
 
+    def test_pool_set_quota(self):
+        p = ceph_utils.BasePool(service='admin', op={
+            'name': 'fake-pool',
+            'max-bytes': 'fake-byte-quota',
+        })
+        p.set_quota()
+        self.check_call.assert_called_once_with([
+            'ceph', '--id', 'admin', 'osd',
+            'pool', 'set-quota', 'fake-pool', 'max_bytes', 'fake-byte-quota'])
+        self.check_call.reset_mock()
+        p = ceph_utils.BasePool(service='admin', op={
+            'name': 'fake-pool',
+            'max-objects': 'fake-object-count-quota',
+        })
+        p.set_quota()
+        self.check_call.assert_called_once_with([
+            'ceph', '--id', 'admin', 'osd',
+            'pool', 'set-quota', 'fake-pool', 'max_objects',
+            'fake-object-count-quota'])
+        self.check_call.reset_mock()
+        p = ceph_utils.BasePool(service='admin', op={
+            'name': 'fake-pool',
+            'max-bytes': 'fake-byte-quota',
+            'max-objects': 'fake-object-count-quota',
+        })
+        p.set_quota()
+        self.check_call.assert_called_once_with([
+            'ceph', '--id', 'admin', 'osd',
+            'pool', 'set-quota', 'fake-pool', 'max_bytes', 'fake-byte-quota',
+            'max_objects', 'fake-object-count-quota'])
+
+    def test_pool_set_compression(self):
+        p = ceph_utils.BasePool(service='admin', op={
+            'name': 'fake-pool',
+            'compression-algorithm': 'lz4',
+        })
+        p.set_compression()
+        self.check_call.assert_called_once_with([
+            'ceph', '--id', 'admin', 'osd', 'pool', 'set', 'fake-pool',
+            'compression_algorithm', 'lz4'])
+        self.check_call.reset_mock()
+        p = ceph_utils.BasePool(service='admin', op={
+            'name': 'fake-pool',
+            'compression-algorithm': 'lz4',
+            'compression-mode': 'fake-mode',
+            'compression-required-ratio': 'fake-ratio',
+            'compression-min-blob-size': 'fake-min-blob-size',
+            'compression-min-blob-size-hdd': 'fake-min-blob-size-hdd',
+            'compression-min-blob-size-ssd': 'fake-min-blob-size-ssd',
+            'compression-max-blob-size': 'fake-max-blob-size',
+            'compression-max-blob-size-hdd': 'fake-max-blob-size-hdd',
+            'compression-max-blob-size-ssd': 'fake-max-blob-size-ssd',
+        })
+        p.set_compression()
+        self.check_call.assert_has_calls([
+            call(['ceph', '--id', 'admin', 'osd', 'pool', 'set', 'fake-pool',
+                  'compression_algorithm', 'lz4']),
+            call(['ceph', '--id', 'admin', 'osd', 'pool', 'set', 'fake-pool',
+                  'compression_mode', 'fake-mode']),
+            call(['ceph', '--id', 'admin', 'osd', 'pool', 'set', 'fake-pool',
+                  'compression_required_ratio', 'fake-ratio']),
+            call(['ceph', '--id', 'admin', 'osd', 'pool', 'set', 'fake-pool',
+                  'compression_min_blob_size', 'fake-min-blob-size']),
+            call(['ceph', '--id', 'admin', 'osd', 'pool', 'set', 'fake-pool',
+                  'compression_min_blob_size_hdd', 'fake-min-blob-size-hdd']),
+            call(['ceph', '--id', 'admin', 'osd', 'pool', 'set', 'fake-pool',
+                  'compression_min_blob_size_ssd', 'fake-min-blob-size-ssd']),
+            call(['ceph', '--id', 'admin', 'osd', 'pool', 'set', 'fake-pool',
+                  'compression_max_blob_size', 'fake-max-blob-size']),
+            call(['ceph', '--id', 'admin', 'osd', 'pool', 'set', 'fake-pool',
+                  'compression_max_blob_size_hdd', 'fake-max-blob-size-hdd']),
+            call(['ceph', '--id', 'admin', 'osd', 'pool', 'set', 'fake-pool',
+                  'compression_max_blob_size_ssd', 'fake-max-blob-size-ssd']),
+        ], any_order=True)
+
     def test_pool_add_cache_tier(self):
         p = ceph_utils.Pool(name='test', service='admin')
         p.add_cache_tier('cacher', 'readonly')
@@ -584,6 +659,8 @@ class CephUtilsTests(TestCase):
             call(['ceph', '--id', 'admin', 'osd', 'pool', 'create',
                   '--pg-num-min=32', 'test',
                   '2048', '2048', 'erasure', 'default']),
+            call(['ceph', '--id', 'admin', 'osd', 'pool',
+                  'set', 'test', 'target_size_ratio', '1.0']),
             call(['ceph', '--id', 'admin', 'osd', 'pool',
                   'application', 'enable', 'test', 'unknown'])
         ])
@@ -603,18 +680,20 @@ class CephUtilsTests(TestCase):
             'm': '1',
             'plugin': 'jerasure'}
         p = ceph_utils.ErasurePool(name='test', service='admin',
-                                   percent_data=100)
+                                   percent_data=100, allow_ec_overwrites=True)
         p.create()
         self.check_call.assert_has_calls([
             call(['ceph', '--id', 'admin', 'osd', 'pool',
                   'create', '--pg-num-min=32', 'test',
                   '2048', '2048', 'erasure', 'default']),
-            call(['ceph', '--id', 'admin', 'osd', 'pool',
-                  'application', 'enable', 'test', 'unknown']),
             call(['ceph', '--id', 'admin', 'osd', 'pool',
                   'set', 'test', 'target_size_ratio', '1.0']),
+            call(['ceph', '--id', 'admin', 'osd', 'pool',
+                  'application', 'enable', 'test', 'unknown']),
             call(['ceph', '--id', 'admin', 'osd', 'pool',
                   'set', 'test', 'pg_autoscale_mode', 'on']),
+            call(['ceph', '--id', 'admin', 'osd', 'pool',
+                  'set', 'test', 'allow_ec_overwrites', 'true']),
         ])
 
     def test_get_erasure_profile_none(self):
@@ -700,39 +779,120 @@ class CephUtilsTests(TestCase):
     @patch.object(ceph_utils, 'erasure_profile_exists')
     def test_create_erasure_profile(self, existing_profile):
         existing_profile.return_value = True
+        self.test_config.set('customize-failure-domain', False)
         self.cmp_pkgrevno.return_value = -1
-        ceph_utils.create_erasure_profile(service='admin', profile_name='super-profile', erasure_plugin_name='jerasure',
-                                          failure_domain='rack', data_chunks=10, coding_chunks=3)
+        ceph_utils.create_erasure_profile(
+            service='admin', profile_name='super-profile', erasure_plugin_name='jerasure',
+            failure_domain='rack', data_chunks=10, coding_chunks=3,
+            device_class='ssd')
 
         cmd = ['ceph', '--id', 'admin', 'osd', 'erasure-code-profile', 'set', 'super-profile',
                'plugin=' + 'jerasure', 'k=' + str(10), 'm=' + str(3),
-               'ruleset-failure-domain=' + 'rack', '--force']
+               'ruleset-failure-domain=' + 'rack',
+               '--force']
         self.check_call.assert_has_calls([call(cmd)])
 
         self.cmp_pkgrevno.return_value = 1
-        ceph_utils.create_erasure_profile(service='admin', profile_name='super-profile', erasure_plugin_name='jerasure',
-                                          failure_domain='rack', data_chunks=10, coding_chunks=3)
+        ceph_utils.create_erasure_profile(
+            service='admin', profile_name='super-profile', erasure_plugin_name='jerasure',
+            failure_domain='rack', data_chunks=10, coding_chunks=3,
+            device_class='ssd')
 
         cmd = ['ceph', '--id', 'admin', 'osd', 'erasure-code-profile', 'set', 'super-profile',
                'plugin=' + 'jerasure', 'k=' + str(10), 'm=' + str(3),
-               'crush-failure-domain=' + 'rack', '--force']
+               'crush-failure-domain=' + 'rack',
+               'crush-device-class=ssd',
+               '--force']
         self.check_call.assert_has_calls([call(cmd)])
 
     @patch.object(ceph_utils, 'erasure_profile_exists')
-    def test_create_erasure_profile_local(self, existing_profile):
+    def test_create_erasure_profile_failure_domain(self, existing_profile):
+        existing_profile.return_value = True
+        self.test_config.set('customize-failure-domain', True)
         self.cmp_pkgrevno.return_value = -1
-        existing_profile.return_value = False
-        ceph_utils.create_erasure_profile(service='admin', profile_name='super-profile', erasure_plugin_name='local',
-                                          failure_domain='rack', data_chunks=10, coding_chunks=3, locality=1)
+        ceph_utils.create_erasure_profile(
+            service='admin', profile_name='super-profile', erasure_plugin_name='jerasure',
+            failure_domain=None, data_chunks=10, coding_chunks=3,
+            device_class='ssd')
 
         cmd = ['ceph', '--id', 'admin', 'osd', 'erasure-code-profile', 'set', 'super-profile',
-               'plugin=' + 'local', 'k=' + str(10), 'm=' + str(3),
-               'ruleset-failure-domain=' + 'rack', 'l=' + str(1)]
+               'plugin=' + 'jerasure', 'k=' + str(10), 'm=' + str(3),
+               'ruleset-failure-domain=' + 'rack',
+               '--force']
+        self.config.assert_called_once_with('customize-failure-domain')
         self.check_call.assert_has_calls([call(cmd)])
 
+    @patch.object(ceph_utils, 'erasure_profile_exists')
+    def test_create_erasure_profile_lrc(self, existing_profile):
+        self.cmp_pkgrevno.return_value = -1
+        self.test_config.set('customize-failure-domain', False)
+        existing_profile.return_value = False
+        ceph_utils.create_erasure_profile(
+            service='admin', profile_name='super-profile', erasure_plugin_name='lrc',
+            failure_domain='host', data_chunks=10, coding_chunks=3, locality=1,
+            crush_locality='rack'
+        )
+
+        cmd = ['ceph', '--id', 'admin', 'osd', 'erasure-code-profile', 'set', 'super-profile',
+               'plugin=' + 'lrc', 'k=' + str(10), 'm=' + str(3),
+               'ruleset-failure-domain=' + 'host', 'l=' + str(1),
+               'crush-locality=rack']
+        self.check_call.assert_has_calls([call(cmd)])
+
+    @patch.object(ceph_utils, 'erasure_profile_exists')
+    def test_create_erasure_profile_lrc_no_locality(self, existing_profile):
+        self.cmp_pkgrevno.return_value = -1
+        self.test_config.set('customize-failure-domain', False)
+        existing_profile.return_value = False
+        self.assertRaises(
+            ValueError,
+            ceph_utils.create_erasure_profile,
+            service='admin', profile_name='super-profile', erasure_plugin_name='lrc',
+            failure_domain='rack', data_chunks=10, coding_chunks=3, locality=None
+        )
+
+    @patch.object(ceph_utils, 'erasure_profile_exists')
+    def test_create_erasure_profile_invalid_plugin(self, existing_profile):
+        self.cmp_pkgrevno.return_value = -1
+        self.test_config.set('customize-failure-domain', False)
+        existing_profile.return_value = False
+        self.assertRaises(
+            AssertionError,
+            ceph_utils.create_erasure_profile,
+            service='admin', profile_name='super-profile', erasure_plugin_name='foobar',
+            data_chunks=10, coding_chunks=3
+        )
+
+    @patch.object(ceph_utils, 'erasure_profile_exists')
+    def test_create_erasure_profile_invalid_technique(self, existing_profile):
+        self.cmp_pkgrevno.return_value = -1
+        self.test_config.set('customize-failure-domain', False)
+        existing_profile.return_value = False
+        self.assertRaises(
+            AssertionError,
+            ceph_utils.create_erasure_profile,
+            service='admin', profile_name='super-profile', erasure_plugin_name='jerasure',
+            data_chunks=10, coding_chunks=3,
+            erasure_plugin_technique='foobar'
+        )
+
+    @patch.object(ceph_utils, 'erasure_profile_exists')
+    def test_create_erasure_profile_invalid_failure_domain(self, existing_profile):
+        self.cmp_pkgrevno.return_value = -1
+        self.test_config.set('customize-failure-domain', False)
+        existing_profile.return_value = False
+        self.assertRaises(
+            AssertionError,
+            ceph_utils.create_erasure_profile,
+            service='admin', profile_name='super-profile', erasure_plugin_name='jerasure',
+            data_chunks=10, coding_chunks=3,
+            failure_domain='foobar',
+        )
+
     @patch.object(ceph_utils, 'erasure_profile_exists')
     def test_create_erasure_profile_shec(self, existing_profile):
         self.cmp_pkgrevno.return_value = -1
+        self.test_config.set('customize-failure-domain', False)
         existing_profile.return_value = False
         ceph_utils.create_erasure_profile(service='admin', profile_name='super-profile', erasure_plugin_name='shec',
                                           failure_domain='rack', data_chunks=10, coding_chunks=3,
@@ -1798,8 +1958,50 @@ class CephUtilsTests(TestCase):
             if os.path.exists(tmpdir):
                 shutil.rmtree(tmpdir)
 
+    def test__partial_build_common_op_create(self):
+        self.maxDiff = None
+        rq = ceph_utils.CephBrokerRq()
+        expect = {
+            'app-name': None,
+            'compression-algorithm': 'lz4',
+            'compression-mode': 'passive',
+            'compression-required-ratio': 0.85,
+            'compression-min-blob-size': 131072,
+            'compression-min-blob-size-hdd': 131072,
+            'compression-min-blob-size-ssd': 8192,
+            'compression-max-blob-size': 524288,
+            'compression-max-blob-size-hdd': 524288,
+            'compression-max-blob-size-ssd': 65536,
+            'group': None,
+            'max-bytes': None,
+            'max-objects': None,
+            'group-namespace': None,
+            'weight': None,
+        }
+        self.assertDictEqual(
+            rq._partial_build_common_op_create(
+                compression_algorithm='lz4',
+                compression_mode='passive',
+                compression_required_ratio=0.85,
+                compression_min_blob_size=131072,
+                compression_min_blob_size_hdd=131072,
+                compression_min_blob_size_ssd=8192,
+                compression_max_blob_size=524288,
+                compression_max_blob_size_hdd=524288,
+                compression_max_blob_size_ssd=65536),
+            expect)
+
     def test_add_op_create_replicated_pool(self):
         base_op = {'app-name': None,
+                   'compression-algorithm': None,
+                   'compression-max-blob-size': None,
+                   'compression-max-blob-size-hdd': None,
+                   'compression-max-blob-size-ssd': None,
+                   'compression-min-blob-size': None,
+                   'compression-min-blob-size-hdd': None,
+                   'compression-min-blob-size-ssd': None,
+                   'compression-mode': None,
+                   'compression-required-ratio': None,
                    'group': None,
                    'group-namespace': None,
                    'max-bytes': None,
@@ -1811,7 +2013,55 @@ class CephUtilsTests(TestCase):
                    'weight': None}
         rq = ceph_utils.CephBrokerRq()
         rq.add_op_create_replicated_pool('apool')
-        self.assertEqual(rq.ops, [base_op])
+        self.assertDictEqual(rq.ops[0], base_op)
+        self.assertRaises(
+            ValueError, rq.add_op_create_pool, 'apool', pg_num=51, weight=100)
+        rq = ceph_utils.CephBrokerRq()
+        self.assertRaises(
+            ValueError,
+            rq.add_op_create_replicated_pool, 'apool',
+            compression_algorithm='invalid')
+        rq = ceph_utils.CephBrokerRq()
+        self.assertRaises(
+            ValueError,
+            rq.add_op_create_replicated_pool, 'apool',
+            compression_mode='invalid')
+        # these parameters should be float / int
+        rq = ceph_utils.CephBrokerRq()
+        self.assertRaises(
+            ValueError,
+            rq.add_op_create_replicated_pool, 'apool',
+            compression_required_ratio='1')
+        rq = ceph_utils.CephBrokerRq()
+        self.assertRaises(
+            ValueError,
+            rq.add_op_create_replicated_pool, 'apool',
+            compression_min_blob_size='1')
+        rq = ceph_utils.CephBrokerRq()
+        self.assertRaises(
+            ValueError,
+            rq.add_op_create_replicated_pool, 'apool',
+            compression_min_blob_size_hdd='1')
+        rq = ceph_utils.CephBrokerRq()
+        self.assertRaises(
+            ValueError,
+            rq.add_op_create_replicated_pool, 'apool',
+            compression_min_blob_size_ssd='1')
+        rq = ceph_utils.CephBrokerRq()
+        self.assertRaises(
+            ValueError,
+            rq.add_op_create_replicated_pool, 'apool',
+            compression_max_blob_size='1')
+        rq = ceph_utils.CephBrokerRq()
+        self.assertRaises(
+            ValueError,
+            rq.add_op_create_replicated_pool, 'apool',
+            compression_max_blob_size_hdd='1')
+        rq = ceph_utils.CephBrokerRq()
+        self.assertRaises(
+            ValueError,
+            rq.add_op_create_replicated_pool, 'apool',
+            compression_max_blob_size_ssd='1')
         rq = ceph_utils.CephBrokerRq()
         rq.add_op_create_pool('apool', replica_count=42)
         op = base_op.copy()
@@ -1855,8 +2105,19 @@ class CephUtilsTests(TestCase):
 
     def test_add_op_create_erasure_pool(self):
         base_op = {'app-name': None,
+                   'allow-ec-overwrites': False,
+                   'compression-algorithm': None,
+                   'compression-max-blob-size': None,
+                   'compression-max-blob-size-hdd': None,
+                   'compression-max-blob-size-ssd': None,
+                   'compression-min-blob-size': None,
+                   'compression-min-blob-size-hdd': None,
+                   'compression-min-blob-size-ssd': None,
+                   'compression-mode': None,
+                   'compression-required-ratio': None,
                    'erasure-profile': None,
                    'group': None,
+                   'group-namespace': None,
                    'max-bytes': None,
                    'max-objects': None,
                    'name': 'apool',
@@ -1865,7 +2126,7 @@ class CephUtilsTests(TestCase):
                    'weight': None}
         rq = ceph_utils.CephBrokerRq()
         rq.add_op_create_erasure_pool('apool')
-        self.assertEqual(rq.ops, [base_op])
+        self.assertDictEqual(rq.ops[0], base_op)
         rq = ceph_utils.CephBrokerRq()
         rq.add_op_create_erasure_pool('apool', weight=42)
         op = base_op.copy()
@@ -1891,3 +2152,49 @@ class CephUtilsTests(TestCase):
         op = base_op.copy()
         op['max-objects'] = 42
         self.assertEqual(rq.ops, [op])
+
+    @patch.object(ceph_utils, 'local_unit')
+    def test_get_previous_request(self, _local_unit):
+        raw_request = CEPH_CLIENT_RELATION['ceph:8']['glance/0']['broker_req']
+        self.relation_get.return_value = raw_request
+        req = ceph_utils.get_previous_request('aRid')
+        self.assertDictEqual(json.loads(req.request), json.loads(raw_request))
+        self.relation_get.assert_called_once_with(
+            attribute='broker_req', rid='aRid', unit=_local_unit())
+
+    @patch.object(ceph_utils, 'uuid')
+    def test_CephBrokerRq__init__(self, _uuid):
+        raw_request = CEPH_CLIENT_RELATION['ceph:8']['glance/0']['broker_req']
+        request = json.loads(raw_request)
+        req1 = ceph_utils.CephBrokerRq(api_version=request['api-version'],
+                                       request_id=request['request-id'])
+        req1.set_ops(request['ops'])
+        req2 = ceph_utils.CephBrokerRq(raw_request_data=raw_request)
+        self.assertDictEqual(
+            json.loads(req1.request),
+            json.loads(req2.request))
+        _uuid.uuid1.return_value = 'fake-uuid'
+        new_req = ceph_utils.CephBrokerRq()
+        expect = {
+            'api-version': 1,
+            'request-id': 'fake-uuid',
+            'ops': [],
+        }
+        self.assertDictEqual(
+            json.loads(new_req.request), expect)
+
+    def test_update_pool(self):
+        ceph_utils.update_pool('aUser', 'aPool', {'aKey': 'aValue'})
+        self.check_call.assert_called_once_with([
+            'ceph', '--id', 'aUser', 'osd', 'pool', 'set',
+            'aPool', 'aKey', 'aValue'])
+        self.check_call.reset_mock()
+        ceph_utils.update_pool('aUser', 'aPool', {
+            'aKey': 'aValue',
+            'anotherKey': 'anotherValue'})
+        self.check_call.assert_has_calls([
+            call(['ceph', '--id', 'aUser', 'osd', 'pool', 'set',
+                  'aPool', 'aKey', 'aValue']),
+            call(['ceph', '--id', 'aUser', 'osd', 'pool', 'set',
+                  'aPool', 'anotherKey', 'anotherValue']),
+        ], any_order=True)
